// Programmer: Maura Dailey

// Class: CSE 120
// Section: 1
// Date: 02/25/02

// Brief: The purpose of this program is to test our modified Queue ADT. Each
//   member function will be tested, and values will be added and subtracted
//   from the queue data type. Detailed commentary should hopefully make the
//   output of each test easy to understand and follow.

// Test Cases: We will test several situations that may occur during the use
// of our Queue ADT. Among them will be the following:

//  1) We will first test the initialization of the queue data type by
//     initializing the data type, then by calling the MakeEmpty() function.
//     Calling the IsEmpty function will determine if the function really has
//     been emptied.
//  2) Next, we need to test our Enqueue function by filling our queue. As we
//     are filling the queue, we will call PrintQueue() to demonstrate that the
//     Enqueue function is working as expected and actually adding data to the
//     queue.
//  3) After that, we call PrintQueue() again to demonstate that the queue contains
//     real data and to verify that PrintQueue() properly displays the data and
//     sends it to a file.
//  4) Now we call the IsFull function to verify that the function correctly
//     recognizes that the queue is now in a full state.
//  5) Now we will empty the queue by calling the function Dequeue until there
//     is no more data. The contents of the queue after removing each value will
//     be displayed to the screen and to an output file through the PrintQueue
//     function.
//  6) Now that the queue has been emptied, we test the IsEmpty function one more
//     time.
//  7) Next, we will fill the queue only partway, using the Enqueue function and
//     the PrintQueue function to display the contents of the queue as you add new
//     items.
//  8) Now we will empty our partly full queue, using the Dequeue function and the
//     PrintQueue function to display the contents of the queue as the data is
//     pulled off the top.

#include <iostream>
#include "QueType.h"
using  namespace std;

int main()
{

  // Declare our queue data type, setting the values to their initial settings.
  QueType queue;
  unsigned int testNum;
  unsigned int temp;

  cout << "We have just initialized our queue, now we will call MakeEmpty()" << endl
       << "to initialize our private member variables." << endl << endl;

  // Empty our queue data type to ensure it is initialized. (Test case 1)
  
  cout << "Calling MakeEmpty now..." << endl << endl;
  queue.MakeEmpty();

  cout << "We will now call the function IsEmpty to determine if it will" << endl
       << "correctly recognize that the queue is now empty." << endl << endl;

  // We will now call IsEmpty to determine if it correctly determines that the queue
  // is empty. (Test case 1)

  cout << "Calling IsEmpty now..." << endl << endl;
  if(queue.IsEmpty())
  {
    cout << "IsEmpty correctly reports that the queue is empty." << endl << endl;
  }
  else
  {
    cout << "IsEmpty incorrectly reports that the queue is not empty." << endl << endl;
  }

  cout << "Now that our queue has been initialized to an empty state, we" << endl
       << "will attempt to fill it to capacity by using the Enqueue function." << endl << endl;

  // We are running a loop that will fill our queue to capacity using the Enqueue()
  // function. Contents are displayed to the screen and to a separate output file
  // using the PrintQueue function. (Test case 2)

  cout << "Testing Enqueue now..." << endl << endl;
  for (testNum = 0; testNum < 8; testNum++)
  {
    if(!(queue.IsFull()))
    {
      queue.Enqueue(testNum);
      queue.PrintQueue();
    }
  }

  cout << endl << "Now that the queue has been filled, the contents will be" << endl
       << "displayed to the screen and sent to an output file as well." << endl << endl;

  // We will now call PrintQueue() to display the full queue. (Test case 3)

  cout << "Calling PrintQueue() now..." << endl << endl;
  queue.PrintQueue();

  cout << endl << "Before we empty the queue, we should determine if the IsFull" << endl
       << "function correctly returns that the queue is now full." << endl << endl;

  // We will now call the IsFull() function to see if it properly is able to determine
  // that the queue is now in a full state. (Test case 4)

  cout << "Calling IsFull() now..." << endl << endl;
  
  if (queue.IsFull())
  {
      cout << "The IsFull function correctly reports that the queue is" << endl
           << "now full." << endl << endl;
  }
  else
  {
    cout << "The IsFull function incorrectly reports that the queue is not" << endl
         << "now full to capacity." << endl << endl;
  }

  cout << "We will now call the Dequeue function until the queue has" << endl
       << "been completely emptied." << endl << endl;

  // We are running a loop that will empty our queue using the Dequeue()
  // function. Contents are displayed to the screen and to a separate output file
  // using the PrintQueue() function. (Test case 5)

  cout << "Testing DeQueue() now..." << endl << endl;
  for (testNum = 0; testNum < 8; testNum++)
  {
    if(!(queue.IsEmpty()))
    {
      queue.Dequeue(temp);
      queue.PrintQueue();
    }
  }

  cout << endl << "Now that Dequeue has emptied our queue, we should again test" << endl
       << "our IsEmpty function to determine if it will correctly determine" << endl
       << "that our queue is now empty." << endl << endl;

  // We will now test the IsEmpty function again, to ensure that it still is able
  // to correctly tell when the queue is empty or not. (Test case 6)

  cout << "Calling IsEmpty() now ..." << endl << endl;
  if (queue.IsEmpty())
  {
    cout << "The IsEmpty function has correctly determined that the queue" << endl
         << "is now empty." << endl << endl;
  }
  else
  {
    cout << "The IsEmpty function has incorrectly determined that the queue" << endl
         << "is not now empty." << endl << endl;
  }

  cout << "Now we will partially fill the queue using our Enqueue function, again" << endl
       << "printing the contents of the queue to the screen with our PrintQueue function" << endl << endl;

  // In this test, we will partially fill our queue to determine that Enqueue and
  // PrintQueue can correctly handle filling and displaying a queue only partway.
  // (Test case 7)

  cout << "Testing Enqueue now..." << endl << endl;
  for (testNum = 0; testNum < 4; testNum++)
  {
    if(!(queue.IsFull()))
    {
      queue.Enqueue(testNum);
      queue.PrintQueue();
    }
  }

  cout << endl << "We will now empty our partially full queue with our Dequeue function." << endl
       << "The contents of the queue will be displayed after each element is removed." << endl << endl;

  // Our final test case empties our partially full queue to ensure that the Dequeue
  // function is correctly able to remove values, even if the queue is not full.
  // (Test case 8)

  cout << "Testing Dequeue now..." << endl << endl;
  for (testNum = 0; testNum < 4; testNum++)
  {
    if(!(queue.IsEmpty()))
    {
      queue.Dequeue(temp);
      queue.PrintQueue();
    }
  }

  cout << endl << "Testing is complete." << endl;

  return 0;
}
