// Programmer: Maura Dailey

// Class: CSE 120
// Section: 1
// Date: 02/25/02

// Implementation file for QueType.h functions

#include "QueType.h"
#include <math.h>
#include <iostream.h>
#include <fstream.h>

QueType::QueType()
{
  // Set the queue to zero, set the index reference value,
  // and set the top of queue index to an initial value.
  queue = 0;
  top = 8;
  index = 8;
}

void QueType::MakeEmpty()
{
  // Reset the value of queue to be zero, and the top of
  // queue index to its initial value.
  queue = 0;
  top = 8;
}

bool QueType::IsEmpty() const
{
  // If the top of queue value is equal to the queue's index
  // value, it must be empty. Else, the queue is not empty.
  return (top == index);
}

bool QueType::IsFull() const
{
  // If the top of queue value is equal to zero, then the
  // queue must be full. Else, the queue is not yet full.
  return (top == 0);
}

void QueType::Enqueue(unsigned int newNode)
{
    // Move the queue to the right to make room for the new queue element.
    queue >>= 4;
    
    // Bitshift the value of the new queue element to make it align with
    // the new opening in the queue.
    newNode <<= 28;
    
    // Add the new queue element to the queue.
    queue += newNode;
    
    // Change the current position of the top element of the queue.
    top--;
}

void QueType::Dequeue(unsigned int& node)
{
  // Assign the starting bit to check based on the current top of the
  // queue.
  unsigned int test_bit = ((top + 1) * 4) - 1;

  unsigned int test_index;
  node = 0;

  for (int i = 3; i >= 0; i--)
  {
    test_index = (test_bit - (3-i));
    
    // Test the current bit to determine if it is a one. If so, raise
    // that bit to the correct 2's complement to obtain its decimal
    // equivalent.
    if ((queue & int(pow(2, test_index)) ) == int(pow(2, test_index)))
    {
      node += int(pow(2, i));
    }
  }

  top++;
}

void QueType::PrintQueue()
{
  // We will use a temporary variable to store top so that we do not
  // destroy the original value as we print queue's contents.
  int temp_top = top;

  unsigned int test_bit;
  unsigned int node;
  unsigned int test_index;

  // Before we perform any operations or create any output, we need to
  // determine if the queue is empty. If it is, we will now exit the
  // function and return to main.
  if (IsEmpty())
    return;

  // Declare the output file stream, and open the file in append mode.
  ofstream file;
  file.open("queue_output.dat", ios::app);

  // Run the following loop until the top of the queue becomes greater than
  // or equal to the index.
  while (temp_top < index)
  {
    // Assign the starting bit to check based on the current top of the queue.
    test_bit = ((temp_top + 1) * 4) - 1;
    node = 0;

    for (int i = 3; i >= 0; i--)
    {
      test_index = (test_bit - (3 - i));
      
      // Test the current bit to determine if it is a one. If so, raise
      // that bit to the correct 2's complement to obtain its decimal
      // equivalent.
      if ((queue & int(pow(2, test_index))) == int(pow(2, test_index)))
      {
        node += int(pow(2, i));
      }
    }

    temp_top++;
    
    // Write the extracted data to the screen and to a file stream.
    cout << node << " ";
    file << node << " ";
  }

  // Output is done: move to the next line, and close the file stream.
  cout << endl;
  file << endl;
  file.close();
}
